{$INCLUDE OPTIONS.INC}
{*****************************************************************************}
{                                                                             }
{          Maps v1.11 Generic Associative Containers for Delphi 2-5           }
{                                                                             }
{                 Copyright (c) 2000 Robert R. Marsh, S.J.                    }
{               & the British Province of the Society of Jesus                }
{                                                                             }
{                                                                             }
{       If you like these Maps and find yourself using then please            }
{       consider making a donation to your favorite charity. I would          }
{       also be pleased if you would acknowledge Maps in any projects         }
{       that make use of them.                                                }
{                                                                             }
{       The Maps Library is supplied as is. The author disclaims all          }
{       warranties, expressed or implied, including, without limitation,      }
{       the warranties of merchantability and of fitness for any              }
{       purpose.                                                              }
{       The author assumes no liability for damages, direct or                }
{       consequential, which may result from use of this code.                }
{                                                                             }
{                            maps@rmarsh.com                                  }
{                        http://www.rmarsh.com                                }
{                                                                             }
{*****************************************************************************}

unit MapTestU;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  StdCtrls,
  ExtCtrls,
  Maps,
  Menus;

type
  TForm1 = class(TForm)
    Memo: TMemo;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Print: TMenuItem;
    N2: TMenuItem;
    Exit: TMenuItem;
    Test1: TMenuItem;
    ListMap: TMenuItem;
    ArrayMap: TMenuItem;
    HATMap: TMenuItem;
    HashMap: TMenuItem;
    TreeMap: TMenuItem;
    TreapMap: TMenuItem;
    SplayMap: TMenuItem;
    AVLMap: TMenuItem;
    N1: TMenuItem;
    Clear: TMenuItem;
    Open: TMenuItem;
    Save: TMenuItem;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    N3: TMenuItem;
    StreamTest: TMenuItem;
    TSTTest1: TMenuItem;
    TSTMap1: TMenuItem;
    procedure OpenClick(Sender: TObject);
    procedure SaveClick(Sender: TObject);
    procedure ClearClick(Sender: TObject);
    procedure PrintClick(Sender: TObject);
    procedure ExitClick(Sender: TObject);
    procedure ListMapClick(Sender: TObject);
    procedure ArrayMapClick(Sender: TObject);
    procedure HATMapClick(Sender: TObject);
    procedure HashMapClick(Sender: TObject);
    procedure TreeMapClick(Sender: TObject);
    procedure TreapMapClick(Sender: TObject);
    procedure SplayMapClick(Sender: TObject);
    procedure AVLMapClick(Sender: TObject);
    procedure StreamTestClick(Sender: TObject);
    procedure TSTTest1Click(Sender: TObject);
    procedure TSTMap1Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure RunTest(m: TMapClass);
    procedure W(line: string);
    procedure WL;
  end;

var
  Form1: TForm1;

implementation

uses
  Variables,
  Printers;
{$R *.DFM}

procedure TForm1.W(line: string);
begin
  Memo.Lines.Add(line);
end;

procedure TForm1.WL;
begin
  Memo.Lines.Add('');
end;

procedure TForm1.RunTest(m: TMapClass);
var
  Map: TMap;
  i: Indicator;
  n: Integer;
  e: extended;

  procedure WriteMap;

    function MyAction(const Index: Indicator): Boolean;
    var
      line: string;
      S: string;
      e: Extended;
    begin
      S := AsString(Map.GetKeyRef(Index));
      e := AsExtended(Map.GetDatumRef(Index));
      line := ^i + S + ^I^I  + FloatToStr(e);
      ShowMessage(line);
      Result := True;
    end;
  var
    i: Indicator;
  begin
    WL;
    i := Map.First;
    while Map.Valid(i) do
    begin
      W(^i + AsString(Map.GetKeyRef(i)) + ^I^I  + FloatToStr(AsExtended(Map.GetDatumRef(i))));
      i := Map.Next;
    end;
    WL;
  end;

begin
  W('Testing  ' + m.Classname + ' ...');
  WL;
  WL;
  W('Addition Test');
  WL;
  Map := m.Create;
  try
    with Map do
    begin
      W('AddItems([''one'', ''two'', ''three''], [1.0, 2.0, 3.0]);');
      AddItems(['one', 'two', 'three'], [1.0, 2.0, 3.0]);
      WriteMap;
      W('ignore duplicates ...');
      WL;
      AcceptDuplicateKeys := False;
      UniqueKeyStrategy := ukIgnore;
      W('AddItems([''one'', ''three''], [1.1, 3.3]);');
      AddItems(['one', 'three'], [1.1, 3.3]);
      WriteMap;
      W('replace duplicates ...');
      WL;
      UniqueKeyStrategy := ukReplace;
      W('AddItems([''one'', ''three''], [1.1, 3.3]);');
      AddItems(['one', 'three'], [1.1, 3.3]);
      WriteMap;
      WL;
      W('Find Test');
      WL;
      W('Find(AsVariable([''two'']), I)');
      WL;
      if Find(AsVariable(['two']), i) then
      begin
        W(^I'two' + ^I^I  + FloatToStr(AsExtended(Map.GetDatumRef(i))));
      end
      else
        W(^I'''two'' not found');
      WL;
      W('Find(AsVariable([''four'']), I)');
      WL;
      if Find(AsVariable(['four']), i) then
      begin
        W(^I'two' + ^I^I  + FloatToStr(AsExtended(Map.GetDatumRef(i))));
      end
      else
        W(^I'''four'''^I^I'not found');
      WL;
      WL;
      W('Deletion Test');
      WL;
      W('DeleteByKey(AsVariable([''three'']))');
      WL;
      DeleteByKey(AsVariable(['three']));
      W('DeleteByKey(AsVariable([''seven'']))');
      WL;
      DeleteByKey(AsVariable(['seven']));
      WriteMap;
      WL;
      W('More Deletions ...');
      WL;
      W('Clear');
      Clear;
      WL;
      W(^I'count' + ^I^I  + IntToStr(Count));
      WL;
      W('for n := 1 to 100 do');
      W('  AddItems([IntToStr(n)], [n*1.0]);');
      for n := 1 to 100 do
        AddItems([IntToStr(n)], [n * 1.0]);
      WL;
      W(^I'count' + ^I^I  + IntToStr(Count));
      WL;
      W('for n := 1 to 50 do');
      W('  DeleteByKey(AsVariable([IntToStr(2*n)]));');
      for n := 1 to 50 do
        DeleteByKey(AsVariable([IntToStr(2 * n)]));
      WL;
      W(^I'count' + ^I^I  + IntToStr(Count));
      WL;
      W('for n := 1 to 50 do');
      W('  DeleteByDatum(AsVariable([2.0*(n-1)+1.0]));');
      for n := 1 to 50 do
        DeleteByDatum(AsVariable([2.0 * (n - 1) + 1.0]));
      WL;
      Clear;
      W(^I'count' + ^I^I  + IntToStr(Count));
      WL;
      W('Array Property test ...');
      WL;
      W('DataByKey[AsVariable([''twenty one''])] := AsVariable([21.0]);');
      DataByKey[AsVariable(['twenty one'])] := AsVariable([21.0]);
      WL;
      W('DataByKey[AsVariable([''twenty two''])] := AsVariable([22.0]);');
      DataByKey[AsVariable(['twenty two'])] := AsVariable([22.0]);
      WL;
      W('DataByKey[AsVariable([''twenty three''])] := AsVariable([23.0]);');
      DataByKey[AsVariable(['twenty three'])] := AsVariable([23.0]);
      WL;
      WriteMap;
      WL;
      e := AsExtended(DataByKey[AsVariable(['twenty three'])]);
      W('e := AsExtended(DataByKey[AsVariable([''twenty three''])]);');
      WL;
      W(^I'e := ' + FloatToStr(e));
      WL;
      W('DataByKey[AsVariable([''twenty three''])] := AsVariable([23.33]);');
      DataByKey[AsVariable(['twenty three'])] := AsVariable([23.33]);
      WL;
      WriteMap;
      WL;
      W('Find(AsVariable([''twenty two'']), I);');
      WL;
      if Find(AsVariable(['twenty two']), i) then
      begin
        W(^I'Data[I] := ' + FloatToStr(AsExtended(Data[i])));
      end;
    end;
  finally
    Map.Free;
  end;
end;

procedure TForm1.OpenClick(Sender: TObject);
begin
  if OpenDialog.Execute then
  begin
    Memo.Lines.LoadFromFile(OpenDialog.FileName);
  end;
end;

procedure TForm1.SaveClick(Sender: TObject);
begin
  if SaveDialog.Execute then
  begin
    Memo.Lines.SaveToFile(SaveDialog.FileName);
  end;
end;

procedure TForm1.ClearClick(Sender: TObject);
begin
  Memo.Lines.Clear;
end;

procedure TForm1.PrintClick(Sender: TObject);
var
  F: TextFile;
  n: Integer;
begin
  AssignPrn(F);
  Rewrite(F);
  for n := 1 to Memo.Lines.Count do
    WriteLn(F, Memo.Lines[n - 1]);
  CloseFile(F);
end;

procedure TForm1.ExitClick(Sender: TObject);
begin
  Close;
end;

procedure TForm1.ListMapClick(Sender: TObject);
begin
  RunTest(TListMap)
end;

procedure TForm1.ArrayMapClick(Sender: TObject);
begin
  RunTest(TArrayMap)
end;

procedure TForm1.HATMapClick(Sender: TObject);
begin
  RunTest(THATMap)
end;

procedure TForm1.HashMapClick(Sender: TObject);
begin
  RunTest(THashMap)
end;

procedure TForm1.TreeMapClick(Sender: TObject);
begin
  RunTest(TTreeMap)
end;

procedure TForm1.TreapMapClick(Sender: TObject);
begin
  RunTest(TTreapMap)
end;

procedure TForm1.SplayMapClick(Sender: TObject);
begin
  RunTest(TSplayMap)
end;

procedure TForm1.AVLMapClick(Sender: TObject);
begin
  RunTest(TAVLMap);
end;

procedure TForm1.TSTMap1Click(Sender: TObject);
begin
  RunTest(TTSTMap);
end;

procedure TForm1.StreamTestClick(Sender: TObject);
var
  m: TMemoryStream;
  r: TVariableStream;
  S: string;
  i: Integer;
  e: extended;
  A: Variant;
begin
  m := TMemoryStream.Create;
  try
    r := TVariableStream.Create(m);
    try
      W('Write the following to a variable stream ...');
      WL;
      r.WriteListBegin;
      W(^I'aardvark');
      r.WriteString('aardvark');
      W(^I'1001');
      r.WriteInteger(1001);
      W(^I'1.0234');
      r.WriteExtended(1.0234);
      W(^I'and a variant array with 5 elements');
      A := VarArrayCreate([0, 4], varVariant);
      W(^I^I'1');
      A[0] := 1;
      W(^I^I'1234.5678');
      A[1] := 1234.5678;
      W(^I^I'Hello world');
      A[2] := 'Hello world';
      W(^I^I'True');
      A[3] := True;
      W(^I^I'VarArrayOf([1, 10, 100, 1000])');
      A[4] := VarArrayOf([1, 10, 100, 1000]);
      r.WriteVariant(A);
      r.WriteListEnd;
      WL;
      r.Position := 0;
      W('... and read them back ...');
      WL;
      r.ReadListBegin;
      r.ReadString(S);
      W(^i + S);
      r.ReadInteger(i);
      W(^i + IntToStr(i));
      r.ReadExtended(e);
      W(^i + FloatToStr(e));
      VarClear(A);
      r.ReadVariant(A);
      W(^I^I+ VarToStr(A[0]));
      W(^I^I+ VarToStr(A[1]));
      W(^I^I+ VarToStr(A[2]));
      W(^I^I+ VarToStr(A[3]));
      W(^I^I+ VarToStr(A[4][0]) + ', ' + VarToStr(A[4][1]) + ', ' + VarToStr(A[4][2]) + ', ' + VarToStr(A[4][3]));
      r.ReadListEnd;
    finally
      r.Free;
    end;
  finally
    m.Free;
  end;
end;

procedure TForm1.TSTTest1Click(Sender: TObject);
var
  F: TextFile;
  Buffer: array[0..1024 * 16 - 1] of Byte;
  Map: TTSTMap;
  Matches: TList;
  i: Indicator;
  n: Integer;
  S: string;
begin
  Map := TTSTMap.Create;
  try
    Matches := TList.Create;
    try
      with Map do
      begin
        AssignFile(F, 'ospd.txt');
        try
          System.SetTextBuf(F, Buffer);
          Reset(F);
          while not Eof(F) do
          begin
            Readln(F, S);
            AddItems([S], [1]);
          end;
        finally
          CloseFile(F);
        end;

        W('Dictionary contains ' + IntToStr(Map.Count) + ' words');
        WL;

        FindWild(AsVariable(['?a?a?a']), Ord('?'), Matches);
        W(IntToStr(Matches.Count) + ' words matching ''?a?a?a'' ...');
        WL;
        for n := 1 to Matches.Count do
        begin
          i := Matches[n - 1];
          W(AsString(GetKeyRef(i)));
        end;
        WL;

        FindNear(AsVariable(['banana']), 2, Matches);
        W(IntToStr(Matches.Count) + ' words 2 characters or less from from ''banana'' ...');
        WL;
        for n := 1 to Matches.Count do
        begin
          i := Matches[n - 1];
          W(AsString(GetKeyRef(i)));
        end;
        WL;

        FindWilder(AsVariable(['*a*a*a']), Ord('?'), Ord('*'), 3, Matches);
        W(IntToStr(Matches.Count) + ' words matching ''*a*a*a'' ...');
        WL;
        for n := 1 to Matches.Count do
        begin
          i := Matches[n - 1];
          W(AsString(GetKeyRef(i)));
        end;
        WL;

      end;
    finally
      Matches.Free;
    end;
  finally
    Map.Free;
  end;

end;

end.

